############################################################
# Author     : Adamson dela Cruz
# Date       : Nov 5, 2017
#
#
# Contains contants for the pricing api
############################################################

# Service
EC2 = 'AmazonEC2'
S3 = 'AmazonS3'
IoT = 'AWSIoT'
RDS = 'AmazonRDS'
LAMBDA = 'AWSLambda'

# Dictionary keys
REGION = 'AWS_REGION'
OFFERS = 'offers'
CURRENT_REGION_INDEX_URL = 'currentRegionIndexUrl'
REGIONS = 'regions'
CURRENT_VERSION_URL = 'currentVersionUrl'

PRODUCT = 'product'
PRODUCTS = 'products'
SKU = 'sku'
ONDEMAND_PRICE = 'ondemand_price'
RESERVED_PRICE = 'reserved_price'
TERMS = 'terms'

ONDEMAND = 'OnDemand'
RESERVED = 'Reserved'
PRICE_DIMENSIONS = 'priceDimensions'
ATTRIBUTES = 'attributes'
INSTANCE_TYPE = 'instanceType'
OPERATING_SYSTEM = 'operatingSystem'

PRICE_PER_UNIT = 'pricePerUnit'
USD = 'USD'
DESCRIPTION = 'description'



########################################################################
# Created By    : Adamson dela Cruz
# Date Created  : Nov 6, 2018
#
# Base class for AWS Offers
########################################################################
from __future__ import absolute_import, division, print_function
from builtins import *
from future.standard_library import install_aliases
install_aliases()

import urllib.request, json

class AWSOffer(object):

    def __init__(self, offers, base_url, region):
        self.base_url = base_url
        self.region = region
        self.offers = offers

    def get_offer(self, offer_url):
        '''
        Returns the offer document.
        '''
        with urllib.request.urlopen(self.base_url + offer_url) as url:
            result = json.loads(url.read().decode())
            return result



############################################################
# Author     : Adamson dela Cruz
# Date       : Nov 6, 2017
#
# Handles request for pricing information on EC2 instances
############################################################
from __future__ import absolute_import, division, print_function
from builtins import *
import json

from offers.offer import AWSOffer
from colorama import Fore, Back, Style
from awspricemanager.constants import *
import time

class EC2Offer(AWSOffer):
    '''
    Handles request for pricing information on EC2 instances.
    '''

    def __init__(self, offers, base_url, region):
        super().__init__(offers, base_url, region)
        self.skus = None
        self.terms = None
        self.selected_skus = []

    def get_offer_terms(self, pricedb, prod):
        '''
        Fetches the pricing information using the price database and the current sku/product
        '''

        sku = prod[PRODUCT][SKU]
        ondemand = None
        reserved = None
        price = None
        prod[ONDEMAND_PRICE] = None
        prod[RESERVED_PRICE] = None

        if sku in pricedb[TERMS][ONDEMAND]:
            ondemand = pricedb[TERMS][ONDEMAND][sku]

        if sku in pricedb[TERMS][RESERVED]:
            reserved = pricedb[TERMS][RESERVED][sku]


        def get_price(term):
            '''
            Fetches the price dimension
            '''
            if term != None:
                keys = list(term.keys())
                for key in keys:
                    priceList = list(term[key][PRICE_DIMENSIONS].keys())
                    for pk in priceList:
                        price = term[key][PRICE_DIMENSIONS][pk]
                        return price

        # On Demand Pricing
        if ondemand != None:
            prod[ONDEMAND_PRICE] = get_price(ondemand)

        # Reserved Pricing
        if reserved != None:
            prod[RESERVED_PRICE] = get_price(reserved)


    def get_offer(self, offer_url):
        '''
        Using the correct offer url (accounting for region), it fetches the product Database
        with the correct pricing and other information
        '''
        pricedb = super().get_offer(offer_url)
        self.skus = {}
        self.terms = {}

        # iterate the product/sku and fetch the correct offer terms (which actually contains the price)
        for product in pricedb[PRODUCTS]:
            prod = {}
            prod[PRODUCT] = pricedb[PRODUCTS][product]
            self.get_offer_terms(pricedb, prod)
            self.skus[product] = prod

        return self.skus

    def get_ec2_price_by_instance(self, instance_type, os='linux'):
        '''
        Using the instance type and operating system, it returns
        pricing information. Assumption is that product database (skus) has been populated.
        '''
        if self.skus == None:
            print(Fore.RED + Style.BRIGHT + 'WARNING:'
            , Fore.YELLOW + Style.NORMAL, 'Product Database not initialized...', Style.RESET_ALL )
            return None

        sku_instance_type = None
        selected_skus = []

        for key in self.skus:
            sku = self.skus[key]
            props = sku[PRODUCT][ATTRIBUTES]
            if props.get(INSTANCE_TYPE) != None:
                sku_instance_type = props[INSTANCE_TYPE]
            else:
                sku_instance_type = ''

            # If we have a match, we collect the sku to return
            if sku_instance_type == instance_type and props[OPERATING_SYSTEM].lower() == os:
                selected_skus.append(sku)

        return selected_skus

        # for sku in self.selected_skus:
        #     props = sku['product']['attributes']
        #     ondemand = sku['ondemand_price']
        #     print(
        #     Fore.YELLOW + Style.BRIGHT
        #     , props['operatingSystem']
        #     , Style.NORMAL + Fore.WHITE
        #     , props['vcpu'], 'CPU'
        #     , props['clockSpeed']
        #     , Fore.GREEN, props['memory']
        #     , props['preInstalledSw']
        #     , Fore.MAGENTA, ondemand['pricePerUnit']['USD']
        #     , ondemand['unit']
        #     , Fore.WHITE, ondemand['description']
        #     , Style.RESET_ALL)
        # end = time.time()
        # print('End:', start)
        # print(Fore.WHITE + Style.BRIGHT ,'Total Time:', Style.NORMAL,end - start, 'seconds', Style.RESET_ALL)



############################################################
# Author     : Adamson dela Cruz
# Date       : Nov 6, 2017
#
#
############################################################
from __future__ import absolute_import, division, print_function
from future.standard_library import install_aliases
from builtins import *
install_aliases()

import boto3, os
import urllib.request, json

from colorama import Fore, Back, Style
from offers.offer import *
from offers.ec2 import *
from awspricemanager.constants import *

base_url = 'https://pricing.us-east-1.amazonaws.com'
price_index = '{}/offers/v1.0/aws/index.json'.format(base_url)
region = os.getenv(REGION)


class PriceManager():
    '''
    Class that handles pricing information on different AWS services
    '''

    def __init__(self):
        '''
        Loads the offer index. Contains the starting point of offers for different AWS services
        '''
        with urllib.request.urlopen(price_index) as url:
            data = json.loads(url.read().decode())
            self.offers = data[OFFERS]


    def get_offer_url(self, offer):
        '''
        From the region index, this method will return the correct price index for the current region
        Example:
            "ap-southeast-2" : {
              "regionCode" : "ap-southeast-2",
              "currentVersionUrl" : "/offers/v1.0/aws/AmazonEC2/20171026015458/ap-southeast-2/index.json"
            }
        '''

        offer_url = None
        with urllib.request.urlopen(base_url + self.offers[offer][CURRENT_REGION_INDEX_URL]) as url:
            self.region_data = json.loads(url.read().decode())
            #self.offers = data['offers']
            if self.region_data != None:
                offer_url = self.region_data[REGIONS][region]

        return offer_url

    def get_ec2_price_by_instance(self, instance_type, os):
        '''
        Returns the price of an EC2 instance using the instance type and operating system
        '''
        ec2_offer = EC2Offer(self.offers, base_url, region)
        offer_url = self.get_offer_url(EC2)
        ec2_offer.skus = ec2_offer.get_offer(offer_url[CURRENT_VERSION_URL])

        return ec2_offer.get_ec2_price_by_instance(instance_type, os)



from setuptools import setup, find_packages

setup(
    name='awspricemanager',
    description='AWS Price Manager - Pulls pricing information from AWS Price List',

    version='0.0.14',

    # Author details
    author='Adamson dela Cruz',
    author_email='adamson.delacruz@gmail.com',
    url='https://bitbucket.org/tochininja/aws-price-api/overview',
    license='BSD 3-Clause License',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Utilities',


        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],

    keywords='aws awspricing',

    #packages=['awspricemanager'],
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),

    # install_requires=[
    #     'boto3','colorama','botocore','future','futures'
    # ],
    # entry_points='''
    #     [console_scripts]
    #     awsiot=commands.awsiot:main
    # ''',
    # entry_points={
    #     'console_scripts': [
    #         'awsiot=commands.awsiot:main',
    #     ],
    # },
)
